#-*- coding: utf-8 -*-
import inspect
import sys

from django.core import exceptions
from django.utils.importlib import import_module


CLASS_PATH_ERROR = 'django-fairepart is unable to interpret settings value for %s. '\
                   '%s should be in the form of a tupple: '\
                   '(\'path.to.models.Class\', \'app_label\').'


def load_class(class_path, setting_name=None):
    """
    Loads a class given a class_path. The setting value may be a string or a
    tuple.

    The setting_name parameter is only there for pretty error output, and
    therefore is optional
    """
    if not isinstance(class_path, basestring):
        try:
            class_path, app_label = class_path
        except:
            if setting_name:
                raise exceptions.ImproperlyConfigured(CLASS_PATH_ERROR % (
                    setting_name, setting_name))
            else:
                raise exceptions.ImproperlyConfigured(CLASS_PATH_ERROR % (
                    'this setting', 'It'))

    try:
        class_module, class_name = class_path.rsplit('.', 1)
    except ValueError:
        if setting_name:
            txt = '%s isn\'t a valid module. Check your %s setting' % (
                class_path, setting_name)
        else:
            txt = '%s isn\'t a valid module.' % class_path
        raise exceptions.ImproperlyConfigured(txt)

    try:
        mod = import_module(class_module)
    except ImportError, e:
        if setting_name:
            txt = 'Error importing backend %s: "%s". Check your %s setting' % (
                class_module, e, setting_name)
        else:
            txt = 'Error importing backend %s: "%s".' % (class_module, e)

        raise exceptions.ImproperlyConfigured(txt)

    try:
        clazz = getattr(mod, class_name)
    except AttributeError:
        if setting_name:
            txt = ('Backend module "%s" does not define a "%s" class. Check'
                   ' your %s setting' % (class_module, class_name,
                                         setting_name))
        else:
            txt = 'Backend module "%s" does not define a "%s" class.' % (
                class_module, class_name)
        raise exceptions.ImproperlyConfigured(txt)
    return clazz


def reraise_as(new_exception_or_type):
    """
    Obtained from https://github.com/dcramer/reraise/blob/master/src/reraise.py
    >>> try:
    >>>     do_something_crazy()
    >>> except Exception:
    >>>     reraise_as(UnhandledException)
    """
    __traceback_hide__ = True  # NOQA

    e_type, e_value, e_traceback = sys.exc_info()

    if inspect.isclass(new_exception_or_type):
        new_type = new_exception_or_type
        new_exception = new_exception_or_type()
    else:
        new_type = type(new_exception_or_type)
        new_exception = new_exception_or_type

    new_exception.__cause__ = e_value

    try:
        raise new_type, new_exception, e_traceback
    finally:
        del e_traceback



from mock import patch

from django.test import TestCase
from django.conf import settings
from django.core.urlresolvers import reverse

from exam.decorators import fixture
from exam.cases import Exam

from fairepart.backends.facebook import FacebookBackend
from fairepart.models import Relation, UserSocialAuth, Invitation
from fairepart.compat import User

from facepy import GraphAPI


class FacebookBackendTests(Exam, TestCase):
    @fixture
    def user(self):
        return User.objects.create_user(username='thoas',
                                        email='florent@ulule.com',
                                        password='$ecret')

    @fixture
    def graph_user(self):
        graph = GraphAPI('%s|%s' % (settings.SOCIAL_AUTH_FACEBOOK_KEY,
                                    settings.SOCIAL_AUTH_FACEBOOK_SECRET))

        return graph.post('%s/accounts/test-users' % settings.SOCIAL_AUTH_FACEBOOK_KEY,
                          installed='true',
                          permissions=','.join(settings.SOCIAL_AUTH_FACEBOOK_SCOPE))

    @fixture
    def real_user_social_auth(self):
        return UserSocialAuth.objects.create(
            user=self.user,
            uid=self.graph_user['id'],
            provider='facebook',
            extra_data={
                'access_token': self.graph_user['access_token']
            }
        )

    @fixture
    def user_social_auth(self):
        return UserSocialAuth.objects.create(
            user=self.user,
            uid='662601795',
            provider='facebook',
            extra_data={
                'access_token': 'fake-one'
            }
        )

    def test_invite_view(self):
        self.client.login(username=self.user.username,
                          password='$ecret')

        response = self.client.get(reverse('fairepart_invite'))

        self.assertTemplateUsed(response, 'fairepart/invite.html')
        self.assertEqual(response.status_code, 200)

    def test_invite_errors(self):
        self.client.login(username=self.user.username,
                          password='$ecret')

        email = 'dummy@localhost.fr'

        Invitation.objects.create(from_user=self.user, email=email)

        response = self.client.post(reverse('fairepart_invite'), data={
            'email': email
        })

        self.assertEqual(response.status_code, 200)

        self.assertIn('email', response.context['form'].errors)

    def test_invite_complete(self):
        self.client.login(username=self.user.username,
                          password='$ecret')

        email = 'dummy@localhost.fr'

        response = self.client.post(reverse('fairepart_invite'), data={
            'email': email
        })

        self.assertRedirects(
            response,
            reverse('fairepart_invite_done'),
            status_code=302,
            target_status_code=200
        )

        qs = Invitation.objects.filter(from_user=self.user, email=email)
        self.assertEqual(qs.count(), 1)

        invitation = qs.get()

        self.assertFalse(invitation.token is None)

    @patch.object(FacebookBackend, 'get_friends')
    def test_import_from_user(self, get_friends):
        get_friends.return_value = {
            'data': [{
                'name': "Alix Heuer",
                'id': "529318992"
            }]
        }

        self.client.login(username=self.user.username,
                          password='$ecret')

        self.user_social_auth

        response = self.client.get(reverse('fairepart_import', args=['facebook']))

        self.assertEqual(response.status_code, 302)

        self.assertEqual(Relation.objects.filter(from_user=self.user, provider='facebook').count(), 1)

    @patch.object(FacebookBackend, 'get_friends')
    def test_import_from_user_with_exisiting(self, get_friends):
        get_friends.return_value = {
            'data': [{
                'name': "Alix Heuer",
                'id': "529318992"
            }]
        }

        self.client.login(username=self.user.username,
                          password='$ecret')

        self.user_social_auth

        alix = User.objects.create_user('alix', 'alix@ulule.com', '$ecret')

        UserSocialAuth.objects.create(user=alix, uid=529318992)

        self.client.get(reverse('fairepart_import', args=['facebook']))

        self.assertEqual(Relation.objects.get(from_user=self.user, provider='facebook').to_user, alix)

    def test_relation_linked(self):
        uid = 529318992

        relation = Relation.objects.create(from_user=self.user, uid=uid, extra_data={
            'name': 'Alix Heuer'
        })

        alix = User.objects.create_user('alix', 'alix@ulule.com', '$ecret')

        UserSocialAuth.objects.create(user=alix, uid=uid)

        relation = Relation.objects.get(pk=relation.pk)

        self.assertEqual(relation.to_user, alix)

    def test_relation_list_view(self):
        self.client.login(username=self.user.username,
                          password='$ecret')

        response = self.client.get(reverse('fairepart_relation_list', args=['facebook', ]))

        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'fairepart/relation_list.html')



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'Invitation'
        db.create_table('fairepart_invitation', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('from_user', self.gf('django.db.models.fields.related.ForeignKey')(related_name='invitations_sent', to=orm['auth.User'])),
            ('to_user', self.gf('django.db.models.fields.related.ForeignKey')(blank=True, related_name='invitations_received', null=True, to=orm['auth.User'])),
            ('email', self.gf('django.db.models.fields.EmailField')(max_length=75)),
            ('text', self.gf('django.db.models.fields.TextField')(null=True, blank=True)),
            ('token', self.gf('django.db.models.fields.CharField')(max_length=40)),
        ))
        db.send_create_signal(u'fairepart', ['Invitation'])


    def backwards(self, orm):
        # Deleting model 'Invitation'
        db.delete_table('fairepart_invitation')


    models = {
        u'auth.group': {
            'Meta': {'object_name': 'Group'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        u'auth.permission': {
            'Meta': {'ordering': "(u'content_type__app_label', u'content_type__model', u'codename')", 'unique_together': "((u'content_type', u'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        u'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('sluggable.fields.SluggableField', [], {'unique': 'True', 'max_length': '50', 'populate_from': 'None'})
        },
        u'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        u'fairepart.invitation': {
            'Meta': {'object_name': 'Invitation'},
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75'}),
            'from_user': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'invitations_sent'", 'to': u"orm['auth.User']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'text': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'to_user': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'invitations_received'", 'null': 'True', 'to': u"orm['auth.User']"}),
            'token': ('django.db.models.fields.CharField', [], {'max_length': '40'})
        },
        u'fairepart.relation': {
            'Meta': {'unique_together': "(('from_user', 'provider', 'uid'),)", 'object_name': 'Relation'},
            'extra_data': ('social.apps.django_app.default.fields.JSONField', [], {'default': "'{}'"}),
            'from_user': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'relations_sent'", 'to': u"orm['auth.User']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'provider': ('django.db.models.fields.CharField', [], {'max_length': '32', 'db_index': 'True'}),
            'to_user': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'relations_received'", 'null': 'True', 'to': u"orm['auth.User']"}),
            'uid': ('django.db.models.fields.CharField', [], {'max_length': '255', 'db_index': 'True'})
        }
    }

    complete_apps = ['fairepart']


from django.conf import settings


BACKENDS = getattr(settings, 'FAIREPART_BACKENDS', (
    'fairepart.backends.facebook.FacebookBackend',
    'fairepart.backends.google.GoogleOAuth2Backend',
))

RELATION_LIST_PAGINATE_BY = getattr(settings, 'FAIREPART_RELATION_LIST_PAGINATE_BY', 5)

GOOGLE_APP_NAME = getattr(settings, 'FAIREPART_GOOGLE_APP_NAME', '')



def get_backends():
    from ..settings import BACKENDS
    from ..utils import load_class

    backends = {}

    for backend_path in BACKENDS:
        backend = load_class(backend_path)
        backends[backend.name] = backend

    return backends



from __future__ import unicode_literals

import logging

from django.views import generic
from django.http import HttpResponseBadRequest, HttpResponseRedirect, HttpResponse
from django.core.urlresolvers import reverse
from django.db.models import Q

from .backends import get_backends
from .models import Relation
from .forms import InvitationForm

from . import settings

logger = logging.getLogger('fairepart')


class ImportView(generic.View):
    http_method_names = ['get']

    def get(self, request, *args, **kwargs):
        backend_name = self.kwargs.get('backend')

        backends = get_backends()

        if backend_name not in backends:
            return HttpResponseBadRequest('%s backend not found' % backend_name)

        backend_class = backends.get(backend_name)

        backend = backend_class()
        backend.import_from_user(request.user)

        return HttpResponseRedirect(reverse('fairepart_relation_list', args=[backend_name, ]))


class RelationListView(generic.ListView):
    model = Relation
    template_name = 'fairepart/relation_list.html'
    paginate_by = settings.RELATION_LIST_PAGINATE_BY
    context_object_name = 'relation_list'

    def get_queryset(self):
        qs = super(RelationListView, self).get_queryset().filter(from_user=self.request.user)

        self.provider = self.kwargs.get('provider', None)

        if self.provider:
            qs = qs.filter(provider=self.provider)

        query = self.request.GET.get('q', None)

        if query:
            qs = qs.filter(Q(name__icontains=query) | Q(name__istartswith=query))

        return qs

    def get_context_data(self, **kwargs):
        context = super(RelationListView, self).get_context_data(**kwargs)
        context['provider'] = self.provider
        context['from_user'] = self.request.user

        for relation in context[self.context_object_name]:
            relation.from_user = self.request.user

        return context

    def get_template_names(self):
        template_names = [self.template_name, ]

        if self.provider:
            template_names = [
                'fairepart/%s_relation_list.html' % self.provider,
            ] + template_names

        return template_names


class InviteView(generic.CreateView):
    form_class = InvitationForm
    template_name = 'fairepart/invite.html'

    def get_form_kwargs(self):
        kwargs = super(InviteView, self).get_form_kwargs()
        kwargs['user'] = self.request.user

        return kwargs

    def form_valid(self, form):
        self.object = form.save()

        if self.request.is_ajax():
            return HttpResponse('Ok', status=201)

        return HttpResponseRedirect(self.get_success_url())

    def get_success_url(self):
        return reverse('fairepart_invite_done')


class InviteDoneView(generic.TemplateView):
    template_name = 'fairepart/invite_done.html'



class BaseFairepartException(Exception):
    pass


class ImportFailed(BaseFairepartException):
    def __init__(self, backend, access_token, *args, **kwargs):
        self.backend = backend
        self.access_token = access_token
        super(ImportFailed, self).__init__(backend, *args, **kwargs)


class ProviderDoesNotExist(BaseFairepartException):
    def __init__(self, backend, *args, **kwargs):
        self.backend = backend
        super(ProviderDoesNotExist, self).__init__(backend, *args, **kwargs)


class MissingParameter(BaseFairepartException):
    def __init__(self, backend, parameter, *args, **kwargs):
        self.backend = backend
        self.parameter = parameter
        super(MissingParameter, self).__init__(backend, *args, **kwargs)



from ..exceptions import ProviderDoesNotExist, MissingParameter
from ..models import Relation, UserSocialAuth


class BaseBackend(object):
    model = Relation

    def get_social_auth(self, user):
        model = self.get_social_model()

        try:
            return model.objects.get(user=user,
                                     provider=self.name)
        except model.DoesNotExist:
            raise ProviderDoesNotExist(self, 'SocialAuth instance does not exists for provider %s' % self.name)

    def get_social_model(self):
        return UserSocialAuth

    def import_from_user(self, user):
        raise NotImplementedError

    def get_relations(self, user):
        return self.model.objects.filter(from_user=user,
                                         provider=self.name)

    def imports(self, data):
        for entry in data:
            instance = self.model(**entry)
            instance.save()

    def get_social_auth_by_uids(self, uids):
        return self.get_social_model().objects.filter(uid__in=uids)

    def get_access_token(self, user):
        social_auth = self.get_social_auth(user)

        extra_data = social_auth.extra_data

        access_token = extra_data.get('access_token', None)

        if not access_token:
            raise MissingParameter(self, 'access_token')

        return access_token



from django import forms
from django.utils.translation import ugettext_lazy as _

from .models import Invitation


class InvitationForm(forms.ModelForm):
    class Meta:
        model = Invitation
        fields = ('email', 'text')

    def __init__(self, *args, **kwargs):
        self.user = kwargs.pop('user', None)

        super(InvitationForm, self).__init__(*args, **kwargs)

    def clean_email(self):
        email = self.cleaned_data['email']

        if Invitation.objects.filter(from_user=self.user, email=email).exists():
            raise forms.ValidationError(_('An invitation for %s already exist') % email)

        return email

    def save(self, *args, **kwargs):
        self.instance.from_user = self.user

        super(InvitationForm, self).save(*args, **kwargs)



from django.conf.urls import patterns, url
from django.contrib.auth.decorators import login_required

from . import views


urlpatterns = patterns(
    '',

    url(r'^import/(?P<backend>[^/]+)/$',
        login_required(views.ImportView.as_view()),
        name='fairepart_import'),

    url(r'^invite/$',
        login_required(views.InviteView.as_view()),
        name='fairepart_invite'),

    url(r'^invite/done/$',
        login_required(views.InviteDoneView.as_view()),
        name='fairepart_invite_done'),

    url(r'^relations/(?P<provider>[^/]+)/(?:(?P<page>[\d]+)/)?$',
        login_required(views.RelationListView.as_view()),
        name='fairepart_relation_list'),
)



from .base import BaseBackend

from atom.http import ProxiedHttpClient

import gdata.contacts.service

from .. import settings
from ..helpers import chunks


class OAuth2Token(object):
    def __init__(self, access_token):
        self.access_token = access_token

    def perform_request(self, *args, **kwargs):
        url = 'https://www.google.com/m8/feeds/contacts/default/full'
        http = ProxiedHttpClient()
        return http.request(
            'GET',
            url,
            headers={
                'Authorization': 'OAuth %s' % self.access_token
            }
        )


class GoogleOAuth2Backend(BaseBackend):
    name = 'google-oauth2'
    title = 'Google'

    def import_from_user(self, user):
        access_token = self.get_access_token(user)

        google = gdata.contacts.service.ContactsService(source=settings.GOOGLE_APP_NAME)
        google.current_token = OAuth2Token(access_token)
        feed = google.GetContactsFeed()

        relations = dict((relation.uid, relation)
                         for relation in self.get_relations(user))

        uids = {}

        for entry in feed.entry:
            if not all([entry.email, entry.title, entry.title.text]):
                continue

            for email in entry.email:
                if email.address in relations:
                    continue

                data = {
                    'from_user': user,
                    'uid': email.address,
                    'name': entry.title.text,
                    'provider': self.name
                }

                uids[email.address] = data

        for uid_list in chunks(uids.keys(), 10):
            socials = self.get_social_auth_by_uids(uid_list)

            for social in socials:
                uids[social.uid]['to_user_id'] = social.user_id

        self.imports(uids.values())



from random import random

from hashlib import sha1 as sha_constructor

from django.db import models
from django.db.models import signals
from django.core.validators import validate_email
from django.core.exceptions import ValidationError

from social.apps.django_app.default.fields import JSONField
from social.apps.django_app.default.models import UID_LENGTH, UserSocialAuth

from .compat import User
from .signals import relation_linked, relation_joined


class RelationManager(models.Manager):
    def contribute_to_class(self, cls, name):
        signals.post_save.connect(self.post_save, sender=cls)

        super(RelationManager, self).contribute_to_class(cls, name)

    def post_save(self, instance, *args, **kwargs):
        if kwargs.get('created', False) and instance.to_user_id:
            relation_linked.send(sender=instance.__class__,
                                 instance=instance,
                                 user=instance.to_user)


class Relation(models.Model):
    name = models.CharField(max_length=255, null=True, blank=True)
    from_user = models.ForeignKey(User, related_name='relations_sent')
    to_user = models.ForeignKey(User,
                                related_name='relations_received',
                                null=True,
                                blank=True)
    provider = models.CharField(max_length=32, db_index=True)
    uid = models.CharField(max_length=UID_LENGTH, db_index=True)
    extra_data = JSONField()

    objects = RelationManager()

    class Meta:
        """Meta data"""
        unique_together = ('from_user', 'provider', 'uid')
        db_table = 'fairepart_relation'

    def is_uid_email(self):
        try:
            validate_email(self.uid)
        except ValidationError:
            return False
        else:
            return True


class Invitation(models.Model):
    from_user = models.ForeignKey(User, related_name='invitations_sent')
    to_user = models.ForeignKey(User,
                                related_name='invitations_received',
                                null=True,
                                blank=True)
    email = models.EmailField()
    text = models.TextField(blank=True, null=True)
    token = models.CharField(max_length=40)

    class Meta:
        db_table = 'fairepart_invitation'

    def save(self, *args, **kwargs):
        if not self.token:
            salt = sha_constructor(str(random())).hexdigest()[:5]

            self.token = sha_constructor(salt + self.email).hexdigest()

        return super(Invitation, self).save(*args, **kwargs)


def handle_user(sender, instance, *args, **kwargs):
    if kwargs.get('created', False) and instance.email:
        relations = Relation.objects.filter(uid=instance.email, to_user__isnull=True)

        for relation in relations:
            relation.to_user = instance

            relation_joined.send(sender=Relation,
                                 instance=relation,
                                 user=instance)

        relations.update(to_user=instance)


def handle_user_social_auth(sender, instance, *args, **kwargs):
    if kwargs.get('created', False) and instance.uid:
        relations = Relation.objects.filter(uid=instance.uid, to_user__isnull=True)

        for relation in relations:
            relation.to_user = instance.user

            relation_joined.send(sender=Relation,
                                 instance=relation,
                                 user=instance.user)

        relations.update(to_user=instance.user)


signals.post_save.connect(handle_user_social_auth, sender=UserSocialAuth)
signals.post_save.connect(handle_user, sender=User)



# -*- coding: utf-8 -*-
import os
from setuptools import setup, find_packages

version = __import__('fairepart').__version__

root = os.path.abspath(os.path.dirname(__file__))

with open(os.path.join(root, 'README.rst')) as f:
    README = f.read()

setup(
    name='django-fairepart',
    version=version,
    description='A generic application to invite your contact from facebook, google, etc.',
    long_description=README,
    author='Florent Messa',
    author_email='florent.messa@gmail.com',
    url='http://github.com/thoas/django-fairepart',
    packages=find_packages(),
    zip_safe=False,
    include_package_data=True,
    install_requires=[
        'facepy',
        'python-social-auth',
        'gdata',
    ],
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Utilities',
    ]
)



import django

__all__ = ['User']

# Django 1.5+ compatibility
if django.VERSION >= (1, 5):
    from django.contrib.auth import get_user_model
    User = get_user_model()

    update_fields = lambda instance, fields: instance.save(update_fields=fields)
else:
    from django.contrib.auth.models import User

    update_fields = lambda instance, fields: instance.save()



import logging

from facepy import GraphAPI, FacepyError

from .base import BaseBackend

from ..exceptions import ImportFailed
from ..utils import reraise_as
from ..helpers import chunks

logger = logging.getLogger('fairepart')


class FacebookBackend(BaseBackend):
    name = 'facebook'
    title = 'Facebook'

    def import_from_user(self, user):
        access_token = self.get_access_token(user)

        relations = dict((relation.uid, relation)
                         for relation in self.get_relations(user))

        try:
            friends = self.get_friends(access_token)
        except FacepyError as e:
            logger.exception(e)

            reraise_as(ImportFailed(self, access_token, e.message))
        else:
            uids = {}

            for friend in friends['data']:
                if friend['id'] in relations:
                    continue

                data = {
                    'from_user': user,
                    'uid': friend['id'],
                    'name': friend['name'],
                    'provider': self.name
                }

                uids[friend['id']] = data

            for uid_list in chunks(uids.keys(), 10):
                socials = self.get_social_auth_by_uids(uid_list)

                for social in socials:
                    uids[social.uid]['to_user_id'] = social.user_id

            self.imports(uids.values())

    def get_friends(self, access_token):
        return GraphAPI(access_token).get('me/friends')
